@@ -40,7 +40,7 @@ from .urls_patterns import URL_ADMIN_SERVER_INFO, URL_ADMIN_CLIENT_AUTHZ_RESOURC
     URL_ADMIN_USER_CLIENT_ROLES_AVAILABLE, URL_ADMIN_USERS, URL_ADMIN_CLIENT_AUTHZ_RESOURCES_ID, URL_ADMIN_CLIENT_AUTHZ_POLICIES, \
     URL_ADMIN_CLIENT_AUTHZ_POLICIES_ID, URL_ADMIN_CLIENT_AUTHZ_PERMISSIONS, URL_ADMIN_CLIENT_AUTHZ_PERMISSIONS_ID, \
     URL_ADMIN_USER_CLIENT_ROLES_COMPOSITE_ID, URL_ADMIN_GROUP_CLIENT_ROLES, USE_ADMIN_CLIENT_AUTHZ_POLICIES_EVAL, \
-    URL_ADMIN_DEFAULT_GROUPS, URL_ADMIN_GROUP_REALM_ROLES, URL_ADMIN_USER_REALM_ROLES_COMPOSITE
+    URL_ADMIN_DEFAULT_GROUPS, URL_ADMIN_GROUP_REALM_ROLES, URL_ADMIN_USER_REALM_ROLES_COMPOSITE, URL_ADMIN_USER_LOGOUT
 
 def serialize(obj):
     """JSON serializer for objects not serializable by default json code"""
@@ -152,6 +152,8 @@ class KeycloakAdmin:
 
         :return: users list
         """
+        if not query:
+            query = {}
         params_path = {"realm-name": self.realm_name}
         data_raw = self.connection.raw_get(URL_ADMIN_USERS.format(**params_path), **query)
         return raise_error_from_response(data_raw, KeycloakGetError)
@@ -249,7 +251,7 @@ class KeycloakAdmin:
         data_raw = self.connection.raw_get(URL_ADMIN_USER.format(**params_path))
         return raise_error_from_response(data_raw, KeycloakGetError)
 
-    def get_user_groups(self, user_id):
+    def get_user_groups(self, user_id, query=None):
         """
         Get user groups Returns a list of groups of which the user is a member
 
@@ -257,8 +259,10 @@ class KeycloakAdmin:
 
         :return: user groups list
         """
+        if not query:
+            query = {}
         params_path = {"realm-name": self.realm_name, "id": user_id}
-        data_raw = self.connection.raw_get(URL_ADMIN_USER_GROUPS.format(**params_path))
+        data_raw = self.connection.raw_get(URL_ADMIN_USER_GROUPS.format(**params_path), **query)
         return raise_error_from_response(data_raw, KeycloakGetError)
 
     def update_user(self, user_id, payload):
@@ -287,6 +291,20 @@ class KeycloakAdmin:
         data_raw = self.connection.raw_delete(URL_ADMIN_USER.format(**params_path))
         return raise_error_from_response(data_raw, KeycloakGetError, expected_code=204)
 
+    def logout(self, user_id):
+        """
+        Logout the user (close all user sessions)
+
+        :param user_id: User id
+
+        :return: Http response
+        """
+        params_path = {"realm-name": self.realm_name, "id": user_id}
+        payload = {"realm": self.realm_name, "user": user_id}
+        data_raw = self.connection.raw_post(URL_ADMIN_USER_LOGOUT.format(**params_path), 
+                                            data=json.dumps(payload))
+        return raise_error_from_response(data_raw, KeycloakGetError, expected_code=204)
+
     def set_user_password(self, user_id, password, temporary=True):
         """
         Set up a password for the user. If temporary is True, the user will have to reset
@@ -512,7 +530,7 @@ class KeycloakAdmin:
                     return self.get_subgroups(subgroup, path)
         return None
 
-    def get_group_members(self, group_id, **query):
+    def get_group_members(self, group_id, query=None):
         """
         Get members by group id. Returns group members
 
@@ -521,6 +539,8 @@ class KeycloakAdmin:
 
         :return: Keycloak server response (UserRepresentation)
         """
+        if not query:
+            query = {}
         params_path = {"realm-name": self.realm_name, "id": group_id}
         data_raw = self.connection.raw_get(URL_ADMIN_GROUP_MEMBERS.format(**params_path), **query)
         return raise_error_from_response(data_raw, KeycloakGetError)
@@ -760,7 +780,7 @@ class KeycloakAdmin:
         data_raw = self.connection.raw_get(URL_ADMIN_CLIENT_AUTHZ_SETTINGS.format(**params_path))
         return data_raw
 
-    def get_client_authz_resources(self, client_id, query):
+    def get_client_authz_resources(self, client_id, query=None):
         """
         Get resources from client.
 
@@ -768,7 +788,8 @@ class KeycloakAdmin:
         http://www.keycloak.org/docs-api/3.3/rest-api/index.html#_clientrepresentation
         :return: Keycloak server response
         """
-
+        if not query:
+            query = {}
         params_path = {"realm-name": self.realm_name, "id": client_id}
         data_raw = self.connection.raw_get(URL_ADMIN_CLIENT_AUTHZ_RESOURCES.format(**params_path), **query)
         return raise_error_from_response(data_raw, KeycloakGetError)
@@ -818,7 +839,7 @@ class KeycloakAdmin:
         data_raw = self.connection.raw_delete(URL_ADMIN_CLIENT_AUTHZ_RESOURCES_ID.format(**params_path))
         return raise_error_from_response(data_raw, KeycloakGetError, expected_code=204)
 
-    def get_client_authz_policy(self, client_id, query, policy_type='role'):
+    def get_client_authz_policy(self, client_id, query=None, policy_type='role'):
         """
         Get policies from client.
 
@@ -826,7 +847,8 @@ class KeycloakAdmin:
         http://www.keycloak.org/docs-api/3.3/rest-api/index.html#_clientrepresentation
         :return: Keycloak server response
         """
-
+        if not query:
+            query = {}
         params_path = {"realm-name": self.realm_name, "id": client_id, "policy-type": policy_type}
         data_raw = self.connection.raw_get(URL_ADMIN_CLIENT_AUTHZ_POLICIES.format(**params_path), **query)
         return raise_error_from_response(data_raw, KeycloakGetError)
@@ -1284,7 +1306,7 @@ class KeycloakAdmin:
         """
         payload = {"resources":[{"type":resource_name}],
                                 "clientId":client_id,
-                                "userId":user_id};
+                                "userId":user_id}
         params_path = {"realm-name": self.realm_name, "id": client_id}
         data_raw = self.connection.raw_post(USE_ADMIN_CLIENT_AUTHZ_POLICIES_EVAL.format(**params_path),
                                               data=json.dumps(payload, default=serialize))
